/*
 * Project name:
     3-Axis Accel Board
 * Copyright:
     (c) MikroElektronika, 2010.
 * Revision History:
     20101122:
       - initial release;
       - 20101129: (RR);
 * Description:
     This project demonstrates Calibration and Reading the Accel Board outputs
     using the ADConversion library.
     On transition to the next stage program waits for user to bring
     the Vcc to RD0 pin.
 * Test configuration:
 
     MCU:                     ac:ATmega16
                              http://www.atmel.com/dyn/resources/prod_documents/doc2466.pdf

     Dev.Board:               ac:EasyAVR6
                              http://www.mikroe.com/eng/products/view/321/easyavr6-development-system/

     Oscillator:              External Clock 08.0000 MHz

     Ext. Modules:            ac:Accel_board (3-Axis)
                              http://www.mikroe.com/eng/products/view/133/three-axis-accelerometer-board/

     SW:                      mikroC PRO for AVR
                              http://www.mikroe.com/eng/products/view/228/mikroc-pro-for-avr/

 * NOTES:
     - When connecting the Vcc and Gnd consult the board schematics -
       if you don't do this properly you may DAMAGE the Accel board !!!
     - Connect the X-out to RA0, Y-out to RA1 and Z-out to RA2.
     - PORTA LEDs and pull-up/down switches = Off
     - pull-up PORTD and set the buttons jumper to GND
*/


// LCD module connections
sbit LCD_RS at PORTD2_bit;
sbit LCD_EN at PORTD3_bit;
sbit LCD_D4 at PORTD4_bit;
sbit LCD_D5 at PORTD5_bit;
sbit LCD_D6 at PORTD6_bit;
sbit LCD_D7 at PORTD7_bit;

sbit LCD_RS_Direction at DDD2_bit;
sbit LCD_EN_Direction at DDD3_bit;
sbit LCD_D4_Direction at DDD4_bit;
sbit LCD_D5_Direction at DDD5_bit;
sbit LCD_D6_Direction at DDD6_bit;
sbit LCD_D7_Direction at DDD7_bit;
// End LCD module connections


// --- LCD Messages ---
const char *msg1="Board to pos    ";
const char *msg2="and press RD0   ";

char positionNo=1;

signed int zeroG_x, zeroG_y, zeroG_z;
signed int oneG_x, oneG_y, oneG_z;
signed int meas_x, meas_y, meas_z;
char text[17];

// --- Copying strings from ROM to RAM
void strConstCpy(char *dest, const char *source) {
  while(*source)
  *dest++ = *source++ ;

  *dest = 0 ;
}

void Measure() {
  meas_x = Adc_Read(0);   // Measure X_out
  meas_y = Adc_Read(1);   // Measure Y_out
  meas_z = Adc_Read(2);   // Measure Z_out
}

void WriteX(signed int wr_x) {

  if (wr_x < 0) {
    Lcd_Chr(1,11,'-');
    wr_x = -wr_x;
  }
  else
    Lcd_Chr(1,11,wr_x/1000+48);

  Lcd_Chr(1,12,(wr_x/100)%10+48);
  Lcd_Chr(1,13,(wr_x/10)%10+48);
  Lcd_Chr(1,14,wr_x%10+48);
}

void WriteY(signed int wr_y) {

  if (wr_y < 0) {
    Lcd_Chr(2,3,'-');
    wr_y = -wr_y;
  }
  else
    Lcd_Chr(2,3,wr_y/1000+48);

  Lcd_Chr(2,4,(wr_y/100)%10+48);
  Lcd_Chr(2,5,(wr_y/10)%10+48);
  Lcd_Chr(2,6,wr_y%10+48);
}

void WriteZ(signed int wr_z) {

  if (wr_z < 0) {
    Lcd_Chr(2,11,'-');
    wr_z = -wr_z;
  }
  else
    Lcd_Chr(2,11,wr_z/1000+48);

  Lcd_Chr(2,12,(wr_z/100)%10+48);
  Lcd_Chr(2,13,(wr_z/10)%10+48);
  Lcd_Chr(2,14,wr_z%10+48);
}

void Calibrate() {

  strConstCpy(text,msg1);         //  1__ Put the Accel board in the position 1 :
  LCD_Out(1,1,text);              //      parallel to earth's surface
  Lcd_Chr(1,13,positionNo++ + 48);//      Z_axis is vertical with Z label UP
  strConstCpy(text,msg2);         //      to measure the Zero Gravity values for X and Y
  LCD_Out(2,1,text);              //      and 1G Z value
  while (PIND.B0 == 1);           //      Loop until RD0 becomes 1

  Measure();
  zeroG_x = meas_x;
  zeroG_y = meas_y;
  oneG_z  = meas_z;
  Delay_ms(1000);


  strConstCpy(text,msg1);         //  2__ Put the Accel board in the position 2 :
  LCD_Out(1,1,text);              //      X_axis is vertical with X label UP
  Lcd_Chr(1,13,positionNo++ + 48);
  strConstCpy(text,msg2);         //      to measure the 1G X value
  LCD_Out(2,1,text);              //      and Zero Gravity value for Z
  while (PIND.B0 == 1);           //      Loop until RD0 becomes 1

  Measure();
  oneG_x = meas_x;
  zeroG_z = meas_z;
  Delay_ms(1000);

  strConstCpy(text,msg1);         //  3__ Put the Accel board in the position 3 :
  LCD_Out(1,1,text);              //      Y_axis is vertical with Y label UP
  Lcd_Chr(1,13,positionNo++ + 48);
  strConstCpy(text,msg2);         //      to measure the 1G Y value
  LCD_Out(2,1,text);
  while (PIND.B0 == 1);           //      Loop until RD0 becomes 1

  Measure();
  oneG_y = meas_y;

  Lcd_Cmd(_LCD_CLEAR);             // Clear LCD
  LCD_Out(1,1,"ZeroG:  X=      "); // Display ZeroG values
  LCD_Out(2,1,"Y=      Z=      ");
  WriteX(zeroG_x);
  WriteY(zeroG_y);
  WriteZ(zeroG_z);
  Delay_ms(3000);

  Lcd_Cmd(_LCD_CLEAR);             // Clear LCD
  LCD_Out(1,1,"OneG:   X=      "); // Display OneG values
  LCD_Out(2,1,"Y=      Z=      ");
  WriteX(oneG_x);
  WriteY(oneG_y);
  WriteZ(oneG_z);
  Delay_ms(3000);
}

void main() {

  DDRA = 0xF8;                     // configure pins as input
  DDRD = 0;

  Lcd_Init();
  Lcd_Cmd(_LCD_CLEAR);
  Lcd_Cmd(_LCD_CURSOR_OFF);

  Calibrate();

  Lcd_Cmd(_LCD_CLEAR);             // Clear LCD
  LCD_Out(1,1,"Read:   X=      "); // Display readings
  LCD_Out(2,1,"Y=      Z=      ");

  while(1) {                       // Unending loop
    Measure();

    WriteX(meas_x - zeroG_x);
    WriteY(meas_y - zeroG_y);
    WriteZ(meas_z - zeroG_z);

    Delay_ms(300);
  }
}