/*
 * Project name:
     3-Axis Accel Board
 * Copyright:
     (c) MikroElektronika, 2010.
 * Revision History:
     20101122:
       - initial release;
       - 20101129: (RR);
 * Description:
     This project demonstrates Calibration and Reading Angles with Accel Board
     using the ADConversion library.
     On transition to the next stage program waits for user to bring
     the Vcc to RD0 pin.
 * Test configuration:
 
     MCU:                 ac:PIC16F887
                          http://ww1.microchip.com/downloads/en/DeviceDoc/41291D.pdf

     Dev.Board:           ac:EasyPIC6
                          http://www.mikroe.com/en/tools/easypic6/

     Oscillator:          External Clock 08.0000 MHz

     Ext. Modules:        ac:Accel_board (3-Axis)
                          http://www.mikroe.com/eng/products/view/133/three-axis-accelerometer-board/

     SW:                  mikroC PRO for PIC
                          http://www.mikroe.com/en/compilers/mikroc/pro/pic/

 * NOTES:
     - When connecting the Vcc and Gnd consult the board schematics -
       if you don't do this properly you may DAMAGE the Accel board !!!
     - Connect the X-out to RA0, Y-out to RA1 and Z-out to RA2.
     - PORTA LEDs and pull-up/down switches = Off
     - pull-down PORTD and set the buttons jumper to Vcc
*/


// LCD module connections
sbit LCD_RS at RB4_bit;
sbit LCD_EN at RB5_bit;
sbit LCD_D4 at RB0_bit;
sbit LCD_D5 at RB1_bit;
sbit LCD_D6 at RB2_bit;
sbit LCD_D7 at RB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

// --- LCD Messages---
const char *msg1="Board to pos    ";
const char *msg2="and press RD0   ";
const char *msg3="ZeroG:  X=      ";
const char *msg4="Y=      Z=      ";
const char *msg5="OneG:   X=      ";
const char *msg6="Angles: X=      ";

char positionNo=1;

signed int x, y, zz;
signed int zeroG_x, zeroG_y, zeroG_z;
signed int oneG_x, oneG_y, oneG_z;
signed int meas_x, meas_y, meas_z;
char text[17];

// --- Copying strings from ROM to RAM
void strConstCpy(char *dest, const char *source) {
  while(*source)
  *dest++ = *source++ ;

  *dest = 0 ;
}

void DoMeasureXYZ() {
  meas_x = Adc_Read(0);   // Measure X_out
  meas_y = Adc_Read(1);   // Measure Y_out
  meas_z = Adc_Read(2);   // Measure Z_out
}

void WriteX(signed int wr_x) {

  if (wr_x < 0) {
    Lcd_Chr(1,11,'-');
    wr_x = -wr_x;
  }
  else
    Lcd_Chr(1,11,wr_x/1000+48);

  Lcd_Chr(1,12,(wr_x/100)%10+48);
  Lcd_Chr(1,13,(wr_x/10)%10+48);
  Lcd_Chr(1,14,wr_x%10+48);
}

void WriteY(signed int wr_y) {

  if (wr_y < 0) {
    Lcd_Chr(2,3,'-');
    wr_y = -wr_y;
  }
  else
    Lcd_Chr(2,3,wr_y/1000+48);

  Lcd_Chr(2,4,(wr_y/100)%10+48);
  Lcd_Chr(2,5,(wr_y/10)%10+48);
  Lcd_Chr(2,6,wr_y%10+48);
}

void WriteZ(signed int wr_z) {

  if (wr_z < 0) {
    Lcd_Chr(2,11,'-');
    wr_z = -wr_z;
  }
  else
    Lcd_Chr(2,11,wr_z/1000+48);

  Lcd_Chr(2,12,(wr_z/100)%10+48);
  Lcd_Chr(2,13,(wr_z/10)%10+48);
  Lcd_Chr(2,14,wr_z%10+48);
}

void Calibrate() {

  strConstCpy(text,msg1);         //  1__ Put the Accel board in the position 1 :
  LCD_Out(1,1,text);              //      parallel to earth's surface
  Lcd_Chr(1,13,positionNo++ + 48);//      Z_axis is vertical with Z label UP
  strConstCpy(text,msg2);         //      to measure the Zero Gravity values for X and Y
  LCD_Out(2,1,text);              //      and 1G Z value
  while (PORTD.B0 == 0);          //      Loop until RD0 becomes 1

  DoMeasureXYZ();
  zeroG_x = meas_x;
  zeroG_y = meas_y;
  oneG_z  = meas_z;
  Delay_ms(1000);


  strConstCpy(text,msg1);         //  2__ Put the Accel board in the position 2 :
  LCD_Out(1,1,text);              //      X_axis is vertical with X label UP
  Lcd_Chr(1,13,positionNo++ + 48);
  strConstCpy(text,msg2);         //      to measure the 1G X value
  LCD_Out(2,1,text);              //      and Zero Gravity value for Z
  while (PORTD.B0 == 0);          //      Loop until RD0 becomes 1

  DoMeasureXYZ();
  oneG_x = meas_x;
  zeroG_z = meas_z;
  Delay_ms(1000);

  strConstCpy(text,msg1);         //  3__ Put the Accel board in the position 3 :
  LCD_Out(1,1,text);              //      Y_axis is vertical with Y label UP
  Lcd_Chr(1,13,positionNo++ + 48);
  strConstCpy(text,msg2);         //      to measure the 1G Y value
  LCD_Out(2,1,text);
  while (PORTD.B0 == 0);          //      Loop until RD0 becomes 1

  DoMeasureXYZ();
  oneG_y = meas_y;

  Lcd_Cmd(_LCD_CLEAR);        // Clear LCD
  strConstCpy(text,msg3);     // Display ZeroG values
  LCD_Out(1,1,text);
  strConstCpy(text,msg4);
  LCD_Out(2,1,text);
  WriteX(zeroG_x);
  WriteY(zeroG_y);
  WriteZ(zeroG_z);
  Delay_ms(3000);

  Lcd_Cmd(_LCD_CLEAR);        // Clear LCD
  strConstCpy(text,msg5);     // Display OneG values
  LCD_Out(1,1,text);
  strConstCpy(text,msg4);
  LCD_Out(2,1,text);
  WriteX(oneG_x);
  WriteY(oneG_y);
  WriteZ(oneG_z);
  Delay_ms(3000);
}

void Read_Angles() {
 double div_x, x_rad, div_y, y_rad, div_z, z_rad ;

  DoMeasureXYZ();
  div_x = (float)(meas_x-zeroG_x) / (oneG_x-zeroG_x);  // measured_G / one_G
  x_rad = asin( div_x );                               // x angle in radians
  x = (x_rad*180) / 3.14;                              // x angle in degrees

  div_y = (float)(meas_y-zeroG_y) / (oneG_y-zeroG_y);  // measured_G / one_G
  y_rad = asin( div_y );                               // y angle in radians
  y = (y_rad*180) / 3.14;                              // y angle in degrees

  div_z = (float)(meas_z-zeroG_z) / (oneG_z-zeroG_z);  // measured_G / one_G
  z_rad = asin( div_z );                               // z angle in radians
  zz = (z_rad*180) / 3.14;                             // z angle in degrees

  WriteX(x);
  WriteY(y);
  WriteZ(zz);
}

void main() {
  ANSEL  = 0x07;              // Configure AN2 pin as analog
  ANSELH = 0;                 // Configure other AN pins as digital I/O
  C1ON_bit = 0;               // Disable comparators
  C2ON_bit = 0;
  TRISA  = 0b00000111;
  PORTD = 0;                  // configure RD0 is digital input
  TRISD = 1;

  Lcd_Init();
  Lcd_Cmd(_LCD_CLEAR);
  Lcd_Cmd(_LCD_CURSOR_OFF);

  Calibrate();

  Lcd_Cmd(_LCD_CLEAR);        // Clear LCD
  strConstCpy(text,msg6);     // Display angles
  LCD_Out(1,1,text);
  strConstCpy(text,msg4);
  LCD_Out(2,1,text);
  //--- main loop
  while(1) {
    Read_Angles();
    Delay_ms(300);
  }
}